from .argument import Argument


class ProgArguments:
    """
    This class is responsible for creating an object containing all arguments used in the program
    """

    def __init__(self):
        self.file_alias = Argument('file_alias',
                                   '-a',
                                   '--file-alias',
                                   'chosen UNIQUE alias for the file. when updating any configurations this flag needs to be used. this only '
                                   'works if the file path already exists in the '
                                   'configurations.',
                                   "")

        self.executable_path = Argument('executable_path',
                                        '-e',
                                        '--executable-path',
                                        'absolute path of file to schedule.',
                                        "")

        self.days_to_schedule = Argument('days_to_schedule',
                                         '-d',
                                         '--days-to-schedule',
                                         'days of the week for when the file will start. multiple values can be set. available set of values: '
                                         '\'monday\', \'tuesday\', \'wednesday\', '
                                         '\'thursday\', \'friday\', \'saturday\' and \'sunday\'.',
                                         "",
                                         default=['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday', 'everyday',
                                                  'weekdays', 'weekends'])

        self.time_to_schedule = Argument('time_to_schedule',
                                         '-t',
                                         '--time-to-schedule',
                                         'input a specific time to start the file. example: \'08:15\'. default value is: \'at startup\'. If is \'at '
                                         'startup\' '
                                         'then the file will be scheduled to open at startup.',
                                         "",
                                         default='at startup')

        self.delete_schedule = Argument('delete_schedule',
                                        '-del',
                                        '--delete-schedule',
                                        'delete a given file alias from the scheduler. example: -d ThisNAmeRefersToCurrentProgramToSchedule',
                                        "")

        self.configure = False

    def to_list(self):
        arguments = [
            self.file_alias,
            self.executable_path,
            self.days_to_schedule,
            self.time_to_schedule,

        ]
        return arguments

    def from_list(self, arguments):
        self.file_alias = arguments[0]
        self.executable_path = arguments[1]
        self.days_to_schedule = arguments[2]
        self.time_to_schedule = arguments[3]



import os
import sys

import yaml

from progscheduler.entities.prog_arguments import ProgArguments
from progscheduler.services.directory import Directory
from progscheduler.services.file import File
from progscheduler.utils.information import show


class Configurations:
    original_arguments = None
    arguments = ProgArguments()
    settings = {}
    path = os.path.dirname(os.path.realpath(__file__)) + '\\userconfigs.yaml'
    file = File(path)

    def set_original_arguments(self, original_arguments):
        self.original_arguments = original_arguments

    def process(self, to_configure):
        self.arguments.configure = to_configure
        self.__check_deletion_of_config()

        if self.is_configured():
            try:
                self.settings = self.__read_yaml()[self.original_arguments.file_alias]
            except (AttributeError, KeyError):
                pass

        if self.arguments.configure:
            self.__configure()

        return self.__get_configs_from_file()

    def is_configured(self):
        directory = Directory(self.file.path)
        if not directory.exists():
            return False

        if self.file.is_empty():
            return False
        return True

    def __configure(self):
        """
        creates a new file and writes all mandatory arguments
        """
        arguments = self.arguments.to_list()

        for configuration in arguments:
            self.settings[configuration.name] = self.__process_configuration(configuration)

        self.__write_to_file()

        self.arguments.from_list(arguments)

        show('The following configuration \n\t\t\t\'' + str(self.arguments.file_alias.value) + '\'\n\t\t was added/updated:\n\t\t\t'
             + str(self.arguments.file_alias.name) + ': ' + str(self.arguments.file_alias.value) + '\n\t\t\t'
             + str(self.arguments.executable_path.name) + ': ' + str(self.arguments.executable_path.value) + '\n\t\t\t'
             + str(self.arguments.days_to_schedule.name) + ': ' + str(self.arguments.days_to_schedule.value) + '\n\t\t\t'
             + str(self.arguments.time_to_schedule.name) + ': ' + str(self.arguments.time_to_schedule.value)
             )

    def __get_configs_from_file(self):
        list_of_configs = self.__read_yaml()
        settings = []

        for config in list_of_configs:
            arguments = ProgArguments()
            list_args = arguments.to_list()

            for arg in list_args:
                arg.set_argument_value(list_of_configs[config][arg.name])

            arguments.from_list(list_args)
            arguments.configure = self.arguments.configure
            settings.append(arguments)

        return settings

    def __process_configuration(self, configuration):
        if configuration.name not in self.original_arguments:
            if not self.is_configured():
                argument_value = configuration.default
            else:
                try:
                    argument_value = self.settings[configuration.name]
                except KeyError:
                    argument_value = configuration.default
        else:
            argument_value = getattr(self.original_arguments, configuration.name)

        configuration.set_argument_value(argument_value)
        return configuration.value

    def __write_to_file(self):
        arguments = self.arguments.to_list()
        list_of_configs = {}
        program_alias = arguments[0].value

        # open all configs from file if configured
        if self.is_configured():
            list_of_configs = self.__read_yaml()

        # create new or update where program alias is current
        list_of_configs[program_alias] = self.settings

        # open file to save new configs
        self.__write_yaml(list_of_configs)

    def __read_yaml(self):
        configs_file = self.file.open('r')
        list_of_configs = yaml.safe_load(configs_file)
        self.file.close()
        return list_of_configs

    def __write_yaml(self, list_of_configs):
        configs_file = self.file.open('w')
        yaml.safe_dump(list_of_configs, configs_file)
        self.file.close()

    def __check_deletion_of_config(self):
        try:
            if self.original_arguments.delete_schedule:
                pass

            if self.is_configured():
                list_of_configs = self.__read_yaml()

                list_of_configs.pop(self.original_arguments.delete_schedule)

                if list_of_configs:
                    self.__write_yaml(list_of_configs)
                else:
                    self.file.open('w').close()

                show('The following configuration \n\t\t\t\'' + self.original_arguments.delete_schedule + '\'\n\t\t was deleted.')

            sys.exit()

        except (AttributeError, KeyError):
            pass



import sys
from progscheduler.arguments import Arguments
from progscheduler.jobs import open_program
from progscheduler.utils.information import show
from .scheduler import Scheduler


def main():
    arguments = Arguments().configure()

    if arguments[0].configure:
        sys.exit()

    show('The program will now start running the scheduler. While this is running this window should not be closed because that will stop the '
         'schedule. If you know that all scheduled jobs are already finished, then it is safe to close this window.')

    scheduler = Scheduler()

    for program in arguments:
        process_scheduler(scheduler, program)

    scheduler.run()


def process_scheduler(scheduler, program_to_schedule):
    scheduler.set_method_to_schedule(lambda: open_program(program_to_schedule.executable_path.value))
    scheduler.process(
        program_to_schedule.days_to_schedule.value,
        program_to_schedule.time_to_schedule.value
    )


if __name__ == '__main__':
    main()



from progscheduler.services.directory import Directory
from progscheduler.entities.prog_arguments import ProgArguments
from progscheduler.configurations.configurations import Configurations
from progscheduler.utils.error import throw
from progscheduler.utils.progsettings import get_version
import argparse


class Arguments:
    """
    This class is responsible for configuring all arguments required for the program to work including saving all mandatory argument values in a
    configuration file for easy frequent use. This way the user only has to configure the first time it runs the program or if the configuration
    file does not exist.
    """

    def __init__(self):
        self.args = None
        self.original_arguments = None
        self.prog_arguments = ProgArguments()
        self.are_configs_saved = False
        self.to_configure = False

    def configure(self):
        """
        create and configure arguments to save in a configuration file
        :return: returns all arguments either from the command line or saved configuration file
        """

        self.args = argparse.ArgumentParser()

        config_file = Configurations()

        self.are_configs_saved = config_file.is_configured()

        self.__add_arguments()

        self.original_arguments = self.args.parse_args()

        self.__check_arguments()

        self.__check_any_errors()

        config_file.set_original_arguments(self.original_arguments)

        return config_file.process(self.to_configure)

    def __add_arguments(self):
        """
        configures and adds the arguments required for the program
        """
        self.args.add_argument('--version', action='version', version='%(prog)s ' + get_version())

        self.args.add_argument(self.prog_arguments.executable_path.abbreviation_name, self.prog_arguments.executable_path.full_name,
                               required=not self.are_configs_saved,
                               help=self.prog_arguments.executable_path.help_message,
                               metavar=self.prog_arguments.executable_path.metavar,
                               default=argparse.SUPPRESS)

        self.args.add_argument(self.prog_arguments.file_alias.abbreviation_name, self.prog_arguments.file_alias.full_name,
                               required=not self.are_configs_saved,
                               help=self.prog_arguments.file_alias.help_message,
                               metavar=self.prog_arguments.file_alias.metavar,
                               default=argparse.SUPPRESS)

        self.args.add_argument(self.prog_arguments.days_to_schedule.abbreviation_name, self.prog_arguments.days_to_schedule.full_name,
                               required=not self.are_configs_saved,
                               nargs='*',
                               choices=self.prog_arguments.days_to_schedule.default,
                               help=self.prog_arguments.days_to_schedule.help_message,
                               metavar=self.prog_arguments.days_to_schedule.metavar,
                               default=argparse.SUPPRESS)

        self.args.add_argument(self.prog_arguments.time_to_schedule.abbreviation_name, self.prog_arguments.time_to_schedule.full_name,
                               type=str,
                               help=self.prog_arguments.time_to_schedule.help_message,
                               metavar=self.prog_arguments.time_to_schedule.metavar,
                               default=self.prog_arguments.time_to_schedule.default)

        self.args.add_argument(self.prog_arguments.delete_schedule.abbreviation_name, self.prog_arguments.delete_schedule.full_name,
                               help=self.prog_arguments.delete_schedule.help_message,
                               metavar=self.prog_arguments.delete_schedule.metavar,
                               default=argparse.SUPPRESS)

    def __check_arguments(self):
        if self.prog_arguments.file_alias.name in self.original_arguments:
            self.to_configure = True

        if self.prog_arguments.days_to_schedule.name in self.original_arguments:
            every_day_of_week = self.prog_arguments.days_to_schedule.default
            if self.original_arguments.days_to_schedule[0] == 'everyday':
                self.original_arguments.days_to_schedule = self.get_specific_days('everyday', every_day_of_week)
            elif self.original_arguments.days_to_schedule[0] == 'weekdays':
                self.original_arguments.days_to_schedule = self.get_specific_days('weekdays', every_day_of_week)
            elif self.original_arguments.days_to_schedule[0] == 'weekends':
                self.original_arguments.days_to_schedule = self.get_specific_days('weekends', every_day_of_week)

    def __check_any_errors(self):
        try:
            if not self.__given_argument_path_exists(self.original_arguments.executable_path):
                throw(self.original_arguments.executable_path + '\' path does not exist.')
        except (AttributeError, TypeError):
            pass

    @staticmethod
    def get_specific_days(days, full_list_of_days):
        if days == 'weekdays':
            full_list_of_days.pop(full_list_of_days.index('saturday'))
            full_list_of_days.pop(full_list_of_days.index('sunday'))
        elif days == 'weekends':
            full_list_of_days.pop(full_list_of_days.index('monday'))
            full_list_of_days.pop(full_list_of_days.index('tuesday'))
            full_list_of_days.pop(full_list_of_days.index('wednesday'))
            full_list_of_days.pop(full_list_of_days.index('thursday'))
            full_list_of_days.pop(full_list_of_days.index('friday'))

        full_list_of_days.pop(full_list_of_days.index('everyday'))
        full_list_of_days.pop(full_list_of_days.index('weekdays'))
        full_list_of_days.pop(full_list_of_days.index('weekends'))

        return full_list_of_days

    @staticmethod
    def __given_argument_path_exists(path):
        argument_path = Directory(path)
        return argument_path.exists()
